#include "matrix.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


error_code matrix_alloc(matrix *mat, int m, int n){

    if (m <= 0 || n <= 0) {
        return err;
    }

    mat->m = m;
    mat->n = n;

    // Allocate memory for the data array
    mat->data = (int **)malloc(m * sizeof(int *));
    if (mat->data == NULL) {
        return err; // Memory allocation failed
    }

    // Allocate memory for each row of the matrix
    for (int i = 0; i < m; i++) {
        mat->data[i] = (int *)malloc(n * sizeof(int));
        if (mat->data[i] == NULL) {
            // If memory allocation fails, free previously allocated memory
            for (int j = 0; j < i; j++) {
                free(mat->data[j]);
            }
            free(mat->data);
            return err; // Memory allocation failed
        }
    }

    return ok; // Success
}



error_code matrix_init(matrix *mat, int m, int n, int val){

    error_code alloc_result = matrix_alloc(mat, m, n);
    if (alloc_result == err) {
        return err; // Memory allocation failed
    }

    for (int i = 0; i < m; i++) {
        for (int j = 0; j < n; j++) { // Corrected the condition i < m to j < n
            mat->data[i][j] = val;
        }
    }

    return ok;
}


error_code matrix_destroy(matrix *mat){

    if (mat->data != NULL) {
        // Free memory for each row
        for (int i = 0; i < mat->m; i++) {
            free(mat->data[i]);
        }
        // Free memory for row pointers
        free(mat->data);
        // Reset matrix fields
        mat->data = NULL;
    }
    // Reset matrix dimensions
    mat->m = -1;
    mat->n = -1;
    return ok;
}

error_code matrix_init_from_array(matrix *mat, int m, int n,int* data)
{
 
    // Allouer de la mémoire pour le tableau 2D de la matrice
    mat->data = (int **)malloc(m * sizeof(int *));
  
    // Allouer de la mémoire pour chaque ligne de la matrice
    for (int i = 0; i < m; i++) {
        mat->data[i] = (int *)malloc(n * sizeof(int));
        if (mat->data[i] == NULL) {
            // Si l'allocation mémoire échoue, libérer la mémoire précédemment allouée
            for (int j = 0; j < i; j++) {
                free(mat->data[j]);
            }
            free(mat->data);
            return err; // Erreur : échec de l'allocation mémoire
        }
    }

    // Copier les données du tableau dans la matrice
    for (int i = 0; i < m; i++) {
        for (int j = 0; j < n; j++) {
            mat->data[i][j] = data[i * n + j];
        }
    }

    // Définir les dimensions de la matrice
    mat->m = m;
    mat->n = n;


    return ok;

}

error_code matrix_clone(matrix *cloned, matrix mat){

    cloned->data = (int **)malloc(mat.m * sizeof(int *));

    
    for (int i = 0; i < mat.m; i++) {
        cloned->data[i] = (int *)malloc(mat.n * sizeof(int));
        if (cloned->data[i] == NULL) {
            // Si l'allocation mémoire échoue, libérer la mémoire précédemment allouée
            for (int j = 0; j < i; j++) {
                free(cloned->data[j]);
            }
            free(cloned->data);
            return err; 
        }

    }
    for (int i = 0; i < mat.m; i++) {
        for(int j=0 ;j< mat.n;i++){

            cloned->data[i][j]= mat.data[i][j];
        }
    }

    cloned->m= mat.m;
    cloned->n = mat.n;


    return ok;
}

error_code matrix_print(matrix mat) {

   
    for (int i = 0; i< mat.m; i++){
        for (int j = 0 ; i< mat.n; j++ ){

            printf("%d ",mat.data[i][j]);
        }
    }

    return ok;
}
error_code matrix_get(int *elem, matrix mat,int ix, int iy){

    // Vérifier si les indices ix et iy sont valides
    if (ix < 0 || ix >= mat.m || iy < 0 || iy >= mat.n) {
        return err; // Erreur : dépassement de capacité
    }

    // Récupérer l'élément de la matrice
    *elem = mat.data[ix][iy];

    return ok; // Succès
}

error_code matrix_set(matrix mat, int ix, int iy, int elem){

    // Vérifier si les indices ix et iy sont valides
    if (ix < 0 || ix >= mat.m || iy < 0 || iy >= mat.n) {
        return err; // Erreur : dépassement de capacité
    }


    mat.data[ix][iy]= elem;


    return ok;
}

error_code pgm_read_from_file(pgm *p,  char *filename) {
    // Open the file in read mode
    FILE *file = fopen(filename, "r");
    if (file == NULL) {
        return err; // Error: failed to open file
    }

    // Buffer for reading lines from the file
    char buffer[256];

    // Read the first line containing PGM magic number
    fgets(buffer, sizeof(buffer), file);

    // Read the next line containing image width, height, and maximum gray value
    int width, height, max_gray;
    fscanf(file, "%d %d %d", &width, &height, &max_gray);

    // Allocate memory for the pixel matrix
    error_code alloc_result = matrix_alloc(&p->pixels, height, width);
    if (alloc_result != ok) {
        fclose(file);
        return alloc_result; // Memory allocation failed
    }

    // Read pixel values
    for (int i = 0; i < height; i++) {
        for (int j = 0; j < width; j++) {
            fscanf(file, "%d", &p->pixels.data[i][j]);
        }
    }

    // Close the file
    fclose(file);

    // Initialize max value in the pgm structure
    p->max = max_gray;

    return ok; // Success
}

int main() {
    // Create a pgm structure
    pgm image;

    // Read the PGM file
    error_code result = pgm_read_from_file(&image, "mandrill.pgm");

    // Check for errors
    if (result != ok) {
        printf("Error reading PGM file. Error code: %d\n", result);
        return err;
    }

    // Print the image information
    printf("Max value: %d\n", image.max);
    printf("Image dimensions: %d x %d\n", image.pixels.m, image.pixels.n);
    printf("Image data:\n");
    for (int i = 0; i < image.pixels.m; i++) {
        for (int j = 0; j < image.pixels.n; j++) {
            printf("%d ", image.pixels.data[i][j]);
        }
        printf("\n");
    }

    // Free memory allocated for the image data
    for (int i = 0; i < image.pixels.m; i++) {
        free(image.pixels.data[i]);
    }
    free(image.pixels.data);

    return 0;
}
